@@ -8,7 +8,7 @@ from asn1editor.controller.Controller import Controller, RootController
 from asn1editor.controller.ControllerFactory import ControllerFactory
 from asn1editor.controller.ListInstanceFactory import ListInstanceFactory
 from asn1editor.view.AbstractView import AbstractView
-from asn1editor.view.AbstractViewFactory import AbstractViewFactory
+from asn1editor.view.AbstractViewFactory import AbstractViewFactory, TypeInfo
 
 
 class ViewControllerFactory(object):
@@ -63,22 +63,23 @@ class ViewControllerFactory(object):
             return self._text(type_, f'ASN.1 type {type_.name} {type_.type_name} not supported')
 
     def _text(self, type_: oer.Type, text: str) -> AbstractView:
-        return self._view_factory.get_text_view(type_.name, text)
+        return self._view_factory.get_text_view(self.__get_type_info(type_), text)
 
     def _null(self, type_: oer.Type, controller: Controller) -> AbstractView:
         ControllerFactory(controller).create_null_controller(type_)
-        return self._view_factory.get_text_view(type_.name, "NULL")
+        return self._view_factory.get_text_view(self.__get_type_info(type_), "NULL")
 
     def _number(self, type_: typing.Union[oer.Integer, oer.Real], checker: constraints_checker.Type, controller: Controller) -> AbstractView:
-        view, value_interface, optional_interface = self._view_factory.get_number_view(type_.name, type_.optional, self.__get_limit(checker.minimum),
-                                                                                       self.__get_limit(checker.maximum), isinstance(type_, oer.Real))
+        view, value_interface, optional_interface = self._view_factory.get_number_view(self.__get_type_info(type_),
+                                                                                       self.__get_limit(checker.minimum), self.__get_limit(checker.maximum),
+                                                                                       isinstance(type_, oer.Real))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface, self.__get_limit(checker.minimum))
 
         return view
 
     def _sequence(self, type_: oer.Sequence, checker: constraints_checker.Dict, controller: Controller) -> AbstractView:
-        view, optional_interface = self._view_factory.get_container_view(type_.name, type_.optional)
+        view, optional_interface = self._view_factory.get_container_view(self.__get_type_info(type_))
 
         sub_controller = ControllerFactory(controller).create_container_controller(type_, optional_interface)
 
@@ -88,8 +89,8 @@ class ViewControllerFactory(object):
         return view
 
     def _sequence_of(self, type_: oer.SequenceOf, checker: constraints_checker.List, controller: Controller) -> AbstractView:
-        view, value_interface, optional_interface = self._view_factory.get_list_view(type_.name, self.__get_limit(checker.minimum),
-                                                                                     self.__get_limit(checker.maximum), type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_list_view(self.__get_type_info(type_), self.__get_limit(checker.minimum),
+                                                                                     self.__get_limit(checker.maximum))
 
         list_instance_factory = ListInstanceFactory(self._view_factory, view, type_.element_type, checker.element_type)
         ControllerFactory(controller).create_list_controller(type_, value_interface, optional_interface, list_instance_factory,
@@ -99,21 +100,21 @@ class ViewControllerFactory(object):
 
     def _enumerated(self, type_: oer.Enumerated, controller: Controller) -> AbstractView:
         choices = [str(value) for value in type_.value_to_data.values()]
-        view, value_interface, optional_interface = self._view_factory.get_enumerated_view(type_.name, choices, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_enumerated_view(self.__get_type_info(type_), choices)
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface)
 
         return view
 
     def _bool(self, type_: oer.Boolean, controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_boolean_view(type_.name, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_boolean_view(self.__get_type_info(type_))
 
         ControllerFactory(controller).create_bool_controller(type_, value_interface, optional_interface)
 
         return view
 
     def _string(self, type_: oer.VisibleString, checker: constraints_checker.String, controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_string_view(type_.name, type_.type_name, type_.optional,
+        view, value_interface, optional_interface = self._view_factory.get_string_view(self.__get_type_info(type_),
                                                                                        self.__get_limit(checker.minimum), self.__get_limit(checker.maximum))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface, self.__get_limit(checker.minimum))
@@ -121,8 +122,8 @@ class ViewControllerFactory(object):
         return view
 
     def _hex_string(self, type_: oer.OctetString, checker: constraints_checker.String, controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_hex_string_view(type_.name, type_.optional, self.__get_limit(checker.minimum),
-                                                                                           self.__get_limit(checker.maximum))
+        view, value_interface, optional_interface = self._view_factory.get_hex_string_view(self.__get_type_info(type_),
+                                                                                           self.__get_limit(checker.minimum), self.__get_limit(checker.maximum))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface, self.__get_limit(checker.minimum))
 
@@ -130,12 +131,12 @@ class ViewControllerFactory(object):
 
     def _bitstring(self, type_: oer.BitString, controller: Controller):
         if type_.number_of_bits is None:
-            view, value_interface, optional_interface = self._view_factory.get_hex_string_view(type_.name, type_.optional, None, None)
+            view, value_interface, optional_interface = self._view_factory.get_hex_string_view(self.__get_type_info(type_), None, None)
 
             ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface, None)
         else:
-            view, value_interface, optional_interface = self._view_factory.get_bitstring_view(type_.name, type_.number_of_bits, type_.named_bits,
-                                                                                              type_.optional)
+            view, value_interface, optional_interface = self._view_factory.get_bitstring_view(self.__get_type_info(type_), type_.number_of_bits,
+                                                                                              type_.named_bits)
 
             ControllerFactory(controller).create_bitstring_controller(type_, value_interface, optional_interface)
 
@@ -143,7 +144,7 @@ class ViewControllerFactory(object):
 
     def _choice(self, type_: oer.Choice, checker: constraints_checker.Choice, controller: Controller):
         choices = [member.name for member in type_.members]
-        view, value_interface, optional_interface = self._view_factory.get_choice_view(type_.name, choices, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_choice_view(self.__get_type_info(type_), choices)
 
         members = {member.name: member for member in type_.members}
         checkers = {member.name: member for member in checker.members}
@@ -154,21 +155,21 @@ class ViewControllerFactory(object):
         return view
 
     def _date(self, type_: oer.Date, controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_date_view(type_.name, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_date_view(self.__get_type_info(type_))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface)
 
         return view
 
     def _time(self, type_: oer.TimeOfDay, controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_time_view(type_.name, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_time_view(self.__get_type_info(type_))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface)
 
         return view
 
     def _datetime(self, type_: typing.Union[oer.DateTime, oer.UTCTime, oer.GeneralizedTime], controller: Controller):
-        view, value_interface, optional_interface = self._view_factory.get_datetime_view(type_.name, type_.optional)
+        view, value_interface, optional_interface = self._view_factory.get_datetime_view(self.__get_type_info(type_))
 
         ControllerFactory(controller).create_value_controller(type_, value_interface, optional_interface)
 
@@ -181,5 +182,21 @@ class ViewControllerFactory(object):
                 return member
 
     @staticmethod
-    def __get_limit(limit: typing.Any):
-        return None if limit in ['MIN', 'MAX'] else limit
+    def __get_limit(limit: typing.Any) -> typing.Optional[int]:
+        return None if limit in ['MIN', 'MAX'] or not isinstance(limit, int) else limit
+
+    @staticmethod
+    def __get_type_info(type_: oer.Type) -> TypeInfo:
+        type_info = TypeInfo()
+        type_info.name = type_.name
+        type_info.optional = type_.optional
+        type_info.tag = f'0x{type_.tag.hex()}' if type_.tag is not None else ''
+        type_to_str = {oer.Integer: 'INTEGER', oer.Real: 'REAL', oer.Enumerated: 'ENUMERATED', oer.Boolean: 'BOOLEAN', oer.OctetString: 'OCTET STRING',
+                       oer.VisibleString: 'VisibleString', oer.UTF8String: 'UTF8String', oer.GeneralString: 'GeneralString', oer.IA5String: 'IA5String',
+                       oer.ObjectIdentifier: 'OBJECT IDENTIFIER', oer.BitString: 'BIT STRING', oer.Sequence: 'SEQUENCE', oer.Set: 'SET',
+                       oer.SequenceOf: 'SEQUENCE OF', oer.SetOf: 'SET OF', oer.Choice: 'CHOICE', oer.Date: 'DATE', oer.TimeOfDay: 'TIME-OF-DAY',
+                       oer.DateTime: 'DATE-TIME', oer.GeneralizedTime: 'GeneralizedTime', oer.UTCTime: 'UTCTime', oer.Null: 'NULL'}
+
+        type_info.typename = type_to_str.get(type(type_), 'UNSUPPORTED: ' + str(type_))
+
+        return type_info
